package Classifying;

import weka.classifiers.Classifier;
import weka.classifiers.trees.RandomForest;

import java.io.File;
import java.io.FileInputStream;
import java.util.ArrayList;

public class ClassifyController {
    private static final String rootM = "/media/Master/ModelFiles/";
    private static final String rootF = "/media/Master/LOS/";
    private static final String rootC = "/media/Master/LOStest/";
    private WebPageClassifier webPageClassifier;

    public ClassifyController(WebPageClassifier webPageClassifier) {
        this.webPageClassifier = webPageClassifier;
    }
    
    public void runLearningOnFolder(String rootPath) {
        byte[] charArray;
        try {
            ArrayList<FilePathAndClass> filePathAndClasses = webPageClassifier.getAllFilePathsFromFolderStructure(rootPath, true);
            for(FilePathAndClass fpt : filePathAndClasses) {
                FileInputStream in = new FileInputStream(fpt.getPath());
                int numChars = in.available();
                charArray = new byte[numChars];
                in.read(charArray);
                in.close();
                webPageClassifier.updateModel(new String(charArray), fpt.getDocClass());
            }
            webPageClassifier.writeModelToFile(rootM, webPageClassifier.getClassifierString());
        }catch (Exception e) {
            e.printStackTrace();
        }
    }


    public void runClassificationOnFolder(String rootPath) {
        byte[] charArray;
        try {
            ArrayList<FilePathAndClass> filePathAndClasses = webPageClassifier.getAllFilePathsFromFolderStructure(rootPath, true);
            for(FilePathAndClass fpt : filePathAndClasses) {
                FileInputStream in = new FileInputStream(fpt.getPath());
                int numChars = in.available();
                charArray = new byte[numChars];
                in.read(charArray);
                in.close();
                webPageClassifier.classifyContent(new String(charArray), fpt.getDocClass());
            }
            webPageClassifier.writeModelToFile(rootM, webPageClassifier.getClassifierString());
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    public void runValidation() {
        try {
            webPageClassifier.performEval();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    public static void main(String[] args) {
        Classifier classifier = new RandomForest();
        String[] options = {"-I", "80", "-K", "1"};
        try {
            WebPageClassifier webPageClassifier = new WebPageClassifier(classifier, options);
            ClassifyController controller = new ClassifyController(webPageClassifier);
            System.out.println("\n<-------- LEARNING -------->\n");
            System.out.println(WebPageClassifier.now());
            controller.runLearningOnFolder(rootF);
            System.out.println("\n<-------- CLASSIFICATION -------->\n");
            File test = new File("/media/Master/TestResults/result_" + controller.webPageClassifier.getClassifierString() + ".txt");
            if(test.exists()) {
                test.renameTo(new File("/media/Master/TestResults/result_" + controller.webPageClassifier.getClassifierString() + ".txt.old"));
                test.delete();
            }
            System.out.println(WebPageClassifier.now());
            controller.runClassificationOnFolder(rootC);
            System.out.println("\n<-------- VALIDATION -------->\n");
            System.out.println(WebPageClassifier.now());
            controller.runValidation();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }
}
